/*
 * @author: pluskid
 * @date:   2008-1-6
 * @brief:  This is a small tool to prevent windows from logging off
            while VirtualBox is still running.

			I like to open a VM in VirtualBox and hide it. Then I use
			ssh client to connect to it or put it in another virtual
			desktop. What I always forgot is to shutdown the VM before
			logging off Windows. VMware has the feature for this situation,
			but VirtualBox has no. So I write this utility.
 */

#include <windows.h>

LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
     static TCHAR szAppName[] = TEXT ("VirtualBox Guard") ;
     HWND         hwnd ;
     MSG          msg ;
     WNDCLASS     wndclass ;

     wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = NULL ;
     wndclass.lpszClassName = szAppName ;

     if (!RegisterClass (&wndclass))
     {
          MessageBox (NULL, TEXT ("VirtuanBox Guard Failed to register window class!"), 
                      szAppName, MB_ICONERROR) ;
          return 0 ;
     }
     hwnd = CreateWindow (szAppName,                  // window class name
                          TEXT ("VirtualBox Guard"),  // window caption
                          WS_OVERLAPPEDWINDOW,        // window style
                          CW_USEDEFAULT,              // initial x position
                          CW_USEDEFAULT,              // initial y position
                          CW_USEDEFAULT,              // initial x size
                          CW_USEDEFAULT,              // initial y size
                          NULL,                       // parent window handle
                          NULL,                       // window menu handle
                          hInstance,                  // program instance handle
                          NULL) ;                     // creation parameters
     
     ShowWindow (hwnd, SW_HIDE/*iCmdShow*/) ;
     UpdateWindow (hwnd) ;

	 /* Be sure our process is shutdown before VirtualBox (and other normal apps)
	    So that we can interpret the shutting down process when needed */
	 SetProcessShutdownParameters (0x3FE, 0);

     while (GetMessage (&msg, NULL, 0, 0))
     {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
     }
     return msg.wParam ;
}

#define TIMER_SHOW_MESSAGE 0xDFF1
VOID CALLBACK WontShutdown(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime)
{
	KillTimer (hwnd, TIMER_SHOW_MESSAGE);
	MessageBox (NULL, TEXT ("Do close all your VirtualBox VMs before logging off!"), 
				TEXT ("WARNING"), MB_ICONWARNING);
}

LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
     switch (message)
     {
	 case WM_QUERYENDSESSION:
		if (FindWindow (TEXT ("QWidget"), TEXT("innotek VirtualBox")))
		{
			SetTimer (hwnd, TIMER_SHOW_MESSAGE, 200, WontShutdown);
			return FALSE;
		}
		else
		{
			return TRUE;
		}
          
     case WM_DESTROY:
          PostQuitMessage (0) ;
          return 0 ;
     }
     return DefWindowProc (hwnd, message, wParam, lParam) ;
}